/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: conn_srvc                                      *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This routine will establish a socket to socket       *
*         connection with a service or port number on a server *
*         using the service name or port number specified in   *
*         the service field of the udp_data structure.         *
*                                                              *
*         There will be a timeout on this entire procedure.    *
*         If we cannot complete a request in a reasonable      *
*         amount of time, (20 seconds) the server is probably  *
*         dead.                                                *
*                                                              *
*       Input:                                                 *
*         The following fields must be setup in the udp_data   *
*         structure passed to this routine.                    *
*                                                              *
*            flags  -- queueing bit and reconnection bit       *
*            host   -- zero terminated string for server name  *
*            svcnm  -- zero terminated string for service name *
*            passwd -- zero terminated string for password     *
*                                                              *
*       Output:                                                *
*         If the connect succeeds, the udp_data->sock field    *
*         will contain the FD for the network socket and the   *
*         routine will return 0.                               *
*                                                              *
*         If the connect succeeds, but we were queued, the     *
*         session will be closed and the routine will return   *
*         a queued status.                                     *
*                                                              *
*         If the connect fails due to a fatal error, i.e. the  *
*         server does not respond, the service does not exist, *
*         etc, the routine will return an error status         *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:
   RKL  15-feb-94  If the target server is down, do not die.
                   Retry the connection.  (Should have been
		   done a long time ago...)

   RKL  12-nov-93  Change to a 60 second timeout to allow 
                   dialup IP routers time to establish a
		   connection.

   RKL  01-nov-93  Removed test_conn flag as it was never used.

   RKL  16-sep-93  Move connect_timeout routine into this module
                   so that it does not have to be declared as an
		   external routine.

   RKL  10-sep-93  Added printing error strings if bad data came
                   back from the server.

   RKL  31-jul-92  Replace bcopy with rtel_copy.  Amazing how
                   many support calls such a small thing can
		   generate.

   RKL  13-jul-92  Added some debugging information and returned
                   correct status from support routines.

   RKL  16-jun-92  Restructure for TLI and SOCKET options.

   RKL  23-mar-92  Enable linger for all machines.  If not enabled,
                   if text comes back from server after rtel software
		   has closed the network file descriptor, the host
		   sends out a reset.  Bad news...

   RKL  05-mar-92  Log error message coming back from server
                   on bad connection status.

   RKL  22-jan-92  Changed return status values to make errors
                   negative and good values positive.

   RKL  03-jan-92  General cleanup.

   KCS  29-oct-91  Got rid of the linger option on netsocket.
                   This is now handled by doing an fflush and
                   before closing the stream.

		   Added ability to form a "telnet" connnection.
		   (i.e. interpret and pad IAC codes)

		   Added the ability to connect to a specified
                   port number rather than to a service.  This
		   is specified by an option in the server_hosts
		   options field.

		   Added to ability to sit and spin if the server's
		   queue is full.  (Note that this is not the host's
		   queue.  This is the queue of pending jobs
		   maintained on the server.)

   KCS  07-may-91  Added message field to errs() call.

   RKL  28-feb-91  Use constant BUFFSIZE for I/O buffers.

   RKL  09-jan-91  Changed RTAP to RTEL.

   RKL  20-nov-90  Added linger option to network socket.  Note that the HP
                   has a totally different interface to the setsockopt routine
		   for turning on lingering...
 */

#include <udpinc.h>

int connect_service(users)
udp_data *users;
{
    char buff[BUFFSIZE], outstr[132];
    int pktlen, stat;

    /* Here we either make a sockets connection, or a TLI connection, based
       on the way the code was compiled.  If the connection was established
       OK, build the request packet and shoot it off to the server.  If we
       got an actual error during the connection attempt, return an error
       status.  If we got refused the connection timed out, assume that the
       server is down and will be coming back up.  Sleep a bit and then try
       the connection again.  */

#ifdef TLI
    while(TRUE){
	stat = open_TLI(users);
	if(stat == OK) break;
	if(stat == PROC_ERR) return(PROC_ERR);
	dbgout("Unable to connect to server.  Sleeping for 60 seconds");
	sleep(60);
    }
    build_conn_packet(users, buff, &pktlen);
    stat = conn_TLI(users, buff, pktlen);
#else
    while(TRUE){
	stat = open_socket(users);
	if(stat == OK) break;
	if(stat == PROC_ERR) return(PROC_ERR);
	dbgout("Unable to connect to server.  Sleeping for 60 seconds");
	sleep(60);
    }
    build_conn_packet(users, buff, &pktlen);
    stat = conn_socket(users, buff, pktlen);
#endif

    /* If the connect succeeded, parse the reply message and return either
       a good status or a queued status.  If the connect failed, close both
       the TCP and UDP sockets and return a fail status. */
    
    if(stat == OK){
	/* At this point, the returned packet will contain the connect reply
	   from the server.  The format of the data returned by the server
	   is as follows:
	   
	   byte 0 -- TYPE_REPL
	   byte 1 -- WILL/WONT code saying whether connection is accepted
	   byte 2 -- WILL returned -- 0 means not queued, 1 means queued
	   WONT returned -- contains failure code
	   
	   If the connect succeeded and we are not queued, return a valid status
	   so actual data transfer can begin.
	   
	   If we are queued, set the queued flag and return a queued status so
	   the calling routine can sleep.
	   
	   If the connect actually failed, close the socket and return a failure
	   status so the calling routine can exit.  */
	
#ifdef DEBUG
	sprintf(dbgbuf, "Solicit Packet response: %2.2X %2.2X %2.2X",
		users->usr_buf[0], users->usr_buf[1], users->usr_buf[2]);
	dbgout(dbgbuf);
#endif
	
	if(users->usr_buf[0] != TYPE_REPL){
	    errs(users, RTEL_BADDATA, "conn_srvc:connect_service");
	    users->usr_buf[40] = 0;
	    sprintf(users->fname, "%s", users->usr_buf);
	    errs(users, RTEL_ERRSTR, "conn_srvc:err_str");
	}
	else{
	    if(users->usr_buf[1] == WILL){
		if(users->usr_buf[2] == QUEUED){
		    close(users->tcp_fd);
		    users->flags |= RETRY;
		    dbgout("Been queued");
		    return(BEENQUEUED);
		}
		else{
		    users->flags |= CONNECTED;
		    dbgout("Connected");
		    return(OK);
		}
	    }
	    else{
		errno = LAT_BASE + users->usr_buf[2];
		errs(users, errno, "conn_srvc:srvrrej");
	    }
	}
    }
    close(users->tcp_fd);
    close(users->udp_fd);
    return(PROC_ERR);
}




int conn_socket(users, buff, pktlen)
udp_data *users;
char     *buff;
int      pktlen;
{
    fd_set efd, rfd;
    int stat;

    dbgout("In conn_socket routine");
    if((stat = write(users->tcp_fd, buff, pktlen)) < OK){
	errs(users, errno, "conn_socket:netwrite");
	return(PROC_ERR);
    };
    
    /* Set up select data structures and wait for a reply from the server */
    
    FD_ZERO(&efd);
    FD_ZERO(&rfd);
    FD_SET(users->tcp_fd, &efd);
    FD_SET(users->tcp_fd, &rfd);

    if((stat = select(getdtablesize(), &rfd, 0, &efd, 0)) < OK){
	errs(users, errno, "conn_socket:badselect");
	return(PROC_ERR);
    }
    
    if(FD_ISSET(users->tcp_fd, &efd)){
	errs(users, RTEL_SELEXCEPT, "conn_socket:selexcept");
	return(PROC_ERR);
    }
    if(FD_ISSET(users->tcp_fd, &rfd)){
	if((stat = read(users->tcp_fd, (char *)users->usr_buf, BUFFSIZE)) <= 0) {
	    errs(users, errno, "conn_socket:conrdfail");
	    return(PROC_ERR);
	}
	return(OK);
    }
    return(PROC_ERR);
}



int build_conn_packet(users, pktptr, pktlen)
udp_data *users;
char     *pktptr;
int      *pktlen;
{
    char buff[MAXLEN], *nmptr, *ptr;
    int i;

    dbgout("In build_conn_packet");
    ptr = pktptr;
    *ptr++ = TYPE_VERS;
    *ptr++ = RTEL_PROT;

    /* Note the strings sent to the server (source host name,
       destination host name, port name, etc...)  cannot be
       more than 17 characters.  (16 actual characters plus
       zero termination. */
    
    *ptr++ = TYPE_SRCHSTNAM;
    gethostname(buff, MAXLEN);
    buff[16] = 0;
    
    if((nmptr = (char *) strchr(buff, '.')) != NULL){
	*nmptr = 0;
    }
    
    *ptr++ = strlen(buff);
    strcpy(ptr, buff);
    ptr += strlen(buff);
    dprintf(dbgbuf, "Source host name -- %s", buff);
    dbgout(dbgbuf);

    /* user supplied destination host name */
    if (users->hnode[0]) {
	users->hnode[16] = 0;
	*ptr++ = TYPE_DSTHSTNAM;
	*ptr++ = strlen(users->hnode);
	strcpy(ptr, users->hnode);
	ptr += strlen(users->hnode);
	dprintf(dbgbuf, "Destination host name -- %s", users->hnode);
	dbgout(dbgbuf);
    }
	
    /* user supplied a port connect variable */
    if (users->hport[0]) {
	users->hport[16] = 0;
	*ptr++ = TYPE_DST_PORT;
	*ptr++ = strlen(users->hport);
	strcpy(ptr, users->hport);
	ptr += strlen(users->hport);
	dprintf(dbgbuf, "Port name -- %s", users->hport);
	dbgout(dbgbuf);
    }

    /* The connection structure passes the UDP port number for replying to
       queued requests, a flag byte saying whether to queue this request
       and if this is a reconnection attempt, a asciz string containing the
       service name and a asciz string containing the password.  */
    
    *ptr++ = TYPE_CONN;
	
    *ptr++ = users->udp_socket & 0xff;
    *ptr++ = (users->udp_socket >> 8) & 0xff;
    
    *ptr++ = users->uniq_id & 0xff;
    *ptr++ = (users->uniq_id >> 8) & 0xff;
    
    *ptr = 0;
    if(users->flags & ALLOW_QUE) *ptr |= QD;
    if(users->flags & RETRY)     *ptr |= RECONNECT;
    ptr++;
    
    users->svcnm[16] = 0;
    *ptr++ = strlen(users->svcnm);
    strcpy(ptr, users->svcnm);
    ptr += strlen(users->svcnm);
    dprintf(dbgbuf, "Service name -- %s", users->svcnm);
    dbgout(dbgbuf);
	
    if(users->passwd[0]) {
	users->passwd[16] = 0;
	*ptr++ = strlen(users->passwd);
	strcpy(ptr, users->passwd);
	ptr += strlen(users->passwd);
	dprintf(dbgbuf, "Password -- %s", users->passwd);
	dbgout(dbgbuf);
    }
    else {
	*ptr++ = 0;
    }
    *pktlen = (int)ptr - (int)pktptr;

#ifdef DEBUG
    dbgout("Solicit Packet contents: ");
    for(i=0; i < *pktlen; i++){
	sprintf(&(dbgbuf[i*3]), "%2.2X ", (unsigned char) *(pktptr + i));
    }
    dbgout(dbgbuf);
#endif

    return(OK);
}



int open_socket(users)
udp_data *users;
{
    struct hostent *gethostbyname();
    struct linger  ling;
    struct hostent *hp;
    
    int len, stat;

    /* First get the server's IP address.  If the server is not in the /etc/hosts
       table, we can't do anything so abort. */

    dbgout("In open_socket routine");
    hp = gethostbyname(users->host);
    if(!hp){
	errs(users, RTEL_HSTNTFND, "conn_srvc:gethstnam");
	return PROC_ERR;
    }
    rtel_copy(hp->h_addr, &(users->tcp_sockdata.sin_addr), hp->h_length);
    
    /* Create a local socket */
    users->tcp_sockdata.sin_family = AF_INET;
    users->tcp_sockdata.sin_port = htons(users->tcp_socket);
    
    if((users->tcp_fd = socket(AF_INET, SOCK_STREAM, TCP)) < OK){
	errs(users, errno, "open_socket:sockerr");
	return(PROC_ERR);
    }
    
    ling.l_onoff = 1;
    ling.l_linger = LING_TIM;
    
    if (setsockopt(users->tcp_fd, SOL_SOCKET, SO_LINGER, (char *) &ling, sizeof(struct linger)) < OK){
        errs(users, errno, "open_socket:setsockopt");
	return(PROC_ERR);
    }
    
    len = sizeof(users->tcp_sockdata);
    if((stat = connect(users->tcp_fd, (struct sockaddr *) &users->tcp_sockdata, len)) < OK){
	dprintf2(dbgbuf, "Server %s not responding, errno %d", users->host, errno);
	dbgout(dbgbuf);
	
	if((errno == ETIMEDOUT) || (errno == ECONNREFUSED)){
	    close(users->tcp_fd);
	    return(RETRY_CONN);
	}
	return(PROC_ERR);
    }
    
    /* The first time through this routine, stuff the original port number
       into the uniq_id field in the users structure.  This number will be
       used for all queueing references until the job completes.

       Then open a UDP socket so that if queueing is enabled, the server
       will know who to send queueing messages to.  Note that the UDP
       channel is opened even if queueing is not enabled, because the
       server always checks the value passed in.  */
    
    dbgout("Getting UDP port");
    if (getsockname(users->tcp_fd, (struct sockaddr *) &(users->temp_sockdata), &len)) {
	errs(users, errno, "open_socket:badsock");
	return(PROC_ERR);
    }
    
    if(!(users->flags & RETRY)) {  /* not a retry connect */
	users->uniq_id = htons(users->temp_sockdata.sin_port);
	if((users->udp_fd = socket(AF_INET, SOCK_DGRAM, UDP)) < OK) {
	    errs(users, errno, "open_socket:cldntsock");
	    return(PROC_ERR);
	}
	
	/* Create name with wildcards.  Start attempting to bind the
	   socket with port number 1024 and keep incrementing until
	   it works... */
	
	users->udp_socket = 1024;
	users->udp_sockdata.sin_family = AF_INET;
	users->udp_sockdata.sin_addr.s_addr = INADDR_ANY;
	users->udp_sockdata.sin_port = htons(users->udp_socket);
	
	while(bind(users->udp_fd, (struct sockaddr *) &users->udp_sockdata, sizeof(users->udp_sockdata)) < 0) {
	    users->udp_socket++;
	    users->udp_sockdata.sin_port = htons(users->udp_socket);
	    if (users->udp_socket > 64000) {
		errs(users, errno,"open_socket:cldntbind");
		return(PROC_ERR);
	    }
	}
	dprintf2(dbgbuf, "Bound to UDP socket: %d, FD: %d", users->udp_socket, users->udp_fd);
	dbgout(dbgbuf);
    }
    return(OK);
}





int open_TLI(users)
udp_data *users;
{    

}
