/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: get_name                                       *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This routine will look for a queue name in the       *
*         server_hosts file.  The queue name to be located is  *
*         based on the name of the current image we are        *
*         executing.  (The correspondence between queues and   *
*         the server/service combination is made by the name   *
*         of the backend filter program executing.)            *
*                                                              *
*       Input:                                                 *
*         This routine requires two input parameters.  The     *
*         first is a character string containing the name of   *
*         the image currently being run.  (This is typically   *
*         argv[0])                                             *
*                                                              *
*         The second parameter is the udp_data structure that  *
*         will be filled in if the specified queue is found.   *
*                                                              *
*       Output:                                                *
*         If the specified queue name is found in the          *
*         server_hosts file, the server, service and password  *
*         fields will be filled in the udp_data structure and  *
*         the routine will return an OK status.                *
*                                                              *
*         If the specified queue name is not found, this       *
*         routine will return an error status.                 *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  16-may-94  Added zeroing service name and port name
                   fields before processing the server_hosts
		   file.

   RKL  19-nov-92  Added parsing formfeed option.

   RKL  31-jan-92  Use a flags bit field rather than individual
                   variables for each option.

   RKL  08-jan-92  Added parsing telnet option.  If telnet is
                   specified, use the "RTEL telnet" socket to
                   connect to ETS.

   RKL  15-may-91  Added extract_queue_name routine for lp/lpr
                   filter programs.

   RKL  14-may-91  Modify first parameter to be queue name to look for
                   rather than argv[0] to allow more generic use of
		   this routine for RTEL II.

   KCS  07-may-91  Added message field to errs() call.

 */

#include <udpinc.h>



/* This routine simply extracts the queue name from the running
   image name (the print queue name for Phase I RTEL) and then
   calls find_queue_name to get information out of the
   server_hosts file. */
   
int extract_queue_name(queue_name, users)
char *queue_name;
udp_data *users;
{  
    char *ptr;
    int stat;
    
    ptr = queue_name + strlen(queue_name);
    
    while((*ptr != '/') && (queue_name != ptr))ptr--;
    if (*ptr == '/') ptr++;
    
    stat = find_queue_name(ptr, users);
    return(stat);
}



/* This routine will find the queue name specified in the server_hosts
   file and place information for the queue into the users data
   structure. */

int find_queue_name(queue_name, users)
char *queue_name;
udp_data *users;
{
    
    FILE *fdp;
    char queue[MAXLEN], option[MAXLEN], service[MAXLEN];
    char *opt1=option, *ptr=service;
    
    if((fdp = fopen(hostfile, "r")) == NULL){
	errs(users, RTEL_BADHSTOPN, "get_name:find_queue_name");
	errs(users, errno, "get_name:find_queue_name");
	return(PROC_ERR);
    }
    
    while(fscanf(fdp, "%s %s %s %s %[ -z]",
		 queue, users->host, service, users->passwd, option) != EOF){

	if (!strcmp(queue_name, queue)) {
	    
	    /* Parse out the service field */
	    while (*ptr && (*ptr != ':'))ptr++;
	    if(*ptr){
		*ptr++ = 0;
		strcpy(users->hport, ptr);
	    }
	    strcpy(users->svcnm, service);
	    
	    /* Parse out password.  If not specified, use none. */
	    if(!strcmp(users->passwd, "null")){
		users->passwd[0] = 0;
	    }
	    
	    /* Parse out options field */
	    while(TRUE){
		if (!strncmp(opt1, "banner", 6)) {
		    users->flags |= OPT_BANNER;
		}
		
		if (!strncmp(opt1, "binary", 6)) {
		    users->flags |= OPT_BINARY;
		}
		
		if (!strncmp(opt1, "expandtabs", 10)) {
		    users->flags |= OPT_EXPTAB;
		}

		if (!strncmp(opt1, "formfeed", 8)) {
		    users->flags |= OPT_FORM;
		}
		
		if (!strncmp(opt1, "postscript", 10)) {
		    users->flags |= OPT_POST;
		    users->flags |= OPT_BINARY;
		}
		
		while (*opt1 && (*opt1 != ',')) opt1++;
		if(*opt1){
		    *opt1++;
		}
		else{
		    fclose(fdp);
		    return(OK);
		}
	    }
	}
    }

    /* If we get here, the name was not found in the queue file. */

    fclose(fdp);
    strcpy(users->host, queue_name);
    errs(users, RTEL_Q_HSTNTFND, "get_name:find_queue_name");
    return(PROC_ERR);
}



find_dev_name(dev_name,users)
char *dev_name;
udp_data *users;
{
    FILE *fdp;
    char dev[MAXLEN], service[MAXLEN];
    char *ptr=service;
    
    if ((fdp = fopen(hostfile, "r")) == NULL){
	errs(users, RTEL_BADHSTOPN, "get_name:find_dev_name");
	errs(users, errno, "get_name:find_dev_name");
	return(PROC_ERR);
    }

    /* Show that we have not found a service name or a port number. */
  
    users->svcnm[0] = 0;
    users->hport[0] = 0;

    /* keep reading till end of file, or found */

    while(fscanf(fdp, "%s %s %s %s %[ -z]",
		 dev, users->host, service, users->passwd, users->option) != EOF){

	if (!strcmp(dev_name, dev)){
	    
	    /* Parse out the service field */
	    while (*ptr && (*ptr != ':'))ptr++;
	    if(*ptr){
		*ptr++ = 0;
		strcpy(users->hport, ptr);
	    }
	    strcpy(users->svcnm, service);
	    
	    /* Parse out password.  If not specified, use none. */
	    if(!strcmp(users->passwd, "null")){
		users->passwd[0] = 0;
	    }
	    fclose(fdp);
	    return(OK);
	}
    }

    /* If we get here, the device name was not found. */
    
    fclose(fdp);
    strcpy(users->host, dev_name);
    errs(users, RTEL_Q_HSTNTFND, "get_name:find_dev_name");
    return(PROC_ERR);
}
