/***************************************************************
*                                                              *
*                   Copyright (c) 1990 - 1995 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: process_file                                   *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This routine will actually move file data to the     *
*         server.                                              *
*                                                              *
*       Input:                                                 *
*         The fd field in the udp_data structure contains the  *
*         file descriptor to use to read from the input        *
*         stream.                                              *
*                                                              *
*       Output:                                                *
*         The data received on the input fd will be sent to    *
*         the server.                                          *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:
   RKL  13-nov-95  Added <CR> before <LF> rather than after it
                   for old printers that are expecting <CR><LF>.
                   Thanks to reader Ron Colcernian with Dynamic
                   Results, Inc.

   RKL  26-jan-94  Add a detector for "job stalled" that will
                   watch for no progress for n seconds and
		   return a status.

   RKL  08-feb-93  Do a bit of casting to allow code to compile
                   cleanly on several more machine.

   RKL  19-nov-92  Added option for sending out form feed and 
                   spin on last character write in case the network
		   pipe is full.

   RKL  07-jan-92  Cleaned up and reorganized data buffer
                   conversion.

   RKL  29-oct-91  Added ifdef for SVR4 type systems around problem
                   case statement.

   KCS  29-oct-91  ditched the pause_abit() call and replaced it with a
                   call to select, providing only the write_fd field, which
		   says to hang until the socket is available to be written
		   on...

   KCS  07-may-91  Added message field to errs() call, and err_handler.

   RKL  02-apr-91  Fix bogus problem with binary type I/O.  Use
                   array of bytes for fread rather than array
		   of blocks.

   RKL  28-feb-91  Use BUFFSIZE constant for file I/O buffer size.

   RKL  10-jan-91  Cleaned up network write return status processing.

   RKL  09-jan-91  Changed RTAP to RTEL.

 */

#include <udpinc.h>

int process_file(users)
udp_data *users;
{
    extern int errno;
    
    char *buf1, *eojstr, tab_buf[BUFFSIZE*8];
    int index, rval, res, readnew, stat, signaled, tabptr, trycnt;
    
    dbgout("In process_file");
    
    /* The input file has been opened by main and the fildesc is in 
       users->fdp.  Also, by this point we should be connected to the
       service, so all we need now is to read and send the file until
       EOF is encountered.  To do this, spin forever in select loop
       and when EOF is reached do a break.
       
       Set the NDELAY option on the network socket.  This will allow
       us to watch for the case where forward progress has stopped and
       do something to inform the user.  

       Note that IBM has to do things differently.  The fcntl call
       does not return an invalid status, but it just doesn't work
       on AIX machines.  */
    
#ifdef AIX
    rval = 1;
    res = ioctl(users->tcp_fd, FIONBIO, &rval);
    if (res < OK){
	errs(users, errno, "process_file:delaynset");
	return(PROC_ERR);
    }
#else
    res = fcntl(users->tcp_fd, F_SETFL, O_NDELAY);
    if (res < OK){
	errs(users, errno, "process_file:delaynset");
	return(PROC_ERR);
    }
#endif    

    readnew = TRUE;
    signaled = FALSE;
    trycnt = 0;
    
    while (TRUE) {
	if(readnew){
	    buf1 = (char *)users->usr_buf;
	    if(users->flags & OPT_BINARY){
		if ((rval = fread(buf1, 1, BUFFSIZE, users->fdp)) == 0) break;
	    }
	    else {
		if (fgets(buf1, BUFFSIZE, users->fdp) == NULL) break;
		rval = strlen(buf1);
	    }
	    
	    /* Now massage the buffer:
	       
	       Tab (0x09) --------------> <VT>
	       <SP> ... <SP>    (expandtabs specified)
	       
	       Line feed (0x0a) --------> <LF>             (binary)
	       <CR> <LF>        (no telnet padding)
	       <CR> <NULL> <LF> (telnet padding)
	       
	       Carriage return (0x0d) --> <CR>             (binary)
	       <CR> <NULL>      (telnet padding)
	       
	       IAC (0xff) --------------> <FF>
	       <FF> <FF>        (telnet padding)
	       
	       All other characters are passed through with no massaging.  */
	    
	    tabptr = 0;
	    for(index=0; index < rval; index++){
		switch ((unsigned char)(*buf1)) {
		    case 0x09: {
			if(users->flags & OPT_EXPTAB){
			    tab_buf[tabptr++] = 0x20;
			    while (((tabptr) & 0x7) != 0) tab_buf[tabptr++] = 0x20;
			    buf1++;
			}
			else {
			    tab_buf[tabptr++] = *buf1++;
			}
			break;
		    }
		    case 0x0a: {
			if(users->flags & OPT_BINARY){
			    tab_buf[tabptr++] = *buf1++;
			}
			else{
			    tab_buf[tabptr++] = 0x0d;
			    tab_buf[tabptr++] = *buf1++;
			}
			break;
		    }
		    default: {
			tab_buf[tabptr++] = *buf1++;
			break;
			}
		}
	    }
	    buf1 = tab_buf;
	    rval = tabptr;
	}
	    
	/* To prevent the network write from hanging indefinitely, the
	   network socket has been set to O_NDELAY which means that if
	   the write would block, the status EWOULDBLOCK or EAGAIN is
	   returned in ERRNO and the write falls through.  If the write
	   does fall through, the sleep statement below should allow the
	   printer time to start flushing the network pipe. */
	
	if ((res = write(users->tcp_fd, buf1, rval)) >= 0) {
	    if (res == rval){
		readnew = TRUE;    /* can read new data */
		trycnt = 0;
		if(signaled){
		    dbgout("Signaling job progress started");
		    signaled = FALSE;
#ifdef AIX
                    log_status(RUNNING);
#endif
		}
	    }
	    else{
		readnew = FALSE;
		buf1 += res;
		rval -= res;
	    }
	}
	else {  /* the write did not succeed at all */
	    switch(errno) {
		case EINTR:{
		    errs(users, errno, "process_file:wrtinterr");
		    break;
		}
		case EWOULDBLOCK:{
		    readnew = FALSE;
		    sleep(1);
		    trycnt++;
		    if(trycnt > NO_PROGRESS){
			if(trycnt > ABORT_JOB){
			    dbgout("Aborting job due to no progress");
			    errs(users, RTEL_JOBHUNG, "process_file:jobhung");
			    return(PROC_ERR);
			}
			if(!signaled){
			    dbgout("Signaling no job progress");
			    signaled = TRUE;
#ifdef AIX
			    log_status(WAITING);
#endif
			}
		    }
		    continue;
		}
		default:{
		    errs(users, errno, "process_file:wrtfailed");
		    rval = PROC_ERR;
		    break;
		}
	    }
	}
    }
    
    /* It is unlikely that this particular write of a single character
       will fail, but sit and spin anyway.  */

    if((users->flags & OPT_POST) || (users->flags & OPT_FORM)){
	if(users->flags & OPT_POST){
	    eojstr = "\004";
	}
	else{
	    eojstr = "\f";
	}
	while((res = write(users->tcp_fd, eojstr, 1)) != 1){
	    if(errno != EWOULDBLOCK){
		errs(users, errno, "process_file:finwrtfail");
		return(PROC_ERR);
	    }
	    sleep(1);
	}
    }
    return OK;
}
