/***************************************************************\
*                                                              *
*                   Copyright (c) 1991 - 1995 Gordian Inc.            *
*                                                              *
*       Author: Rich Lyman                                     *
*       Module: process_pipe.c                                 *
*       Created: Thurs 9 May 10:17:51 1993                     *
*       Abstract:                                              *
*         This module handles the actual communication between *
*         the pipe and the terminal server.                    *
*                                                              *
*         This routine waits for input from the pipe.  When    *
*         data arrives, it establishes a network connection to *
*         the server, and starts moving data.  When the pipe   *
*         read detects and EOF, close the network sockets,     *
*         and start spinning waiting for data to show up       *
*                                                              *
*       Input:                                                 *
*         As specified per routine.                            *
*                                                              *
*       Output:                                                *
*         User data will be moved between the host system and  *
*         a port on the terminal server.  If errors occur,     *
*         they will be logged in the /usr/spool/rtel status    *
*         file.  (job_status)                                  *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:
   RKL  23-oct-95  RISC/OS does not return the correct status
                   when the network would block.  Redefine
                   EWOULDBLOCK to what it actually returns.

   RKL  22-aug-95  Didn't think about if a user writes a zero
                   byte message into the pipe as the very first
                   message.

   RKL  18-aug-95  Rearrange debugging info to make things a
                   bit clearer.

   RKL  17-feb-94  Change processing to retry connections if
                   things go wrong and check for a "stalled"
                   job.

   RKL  12-nov-93  Reset flag field before start of each pipe
                   job.

   RKL  01-nov-93  Removed test_conn option as it was never used.

   RKL  11-may-93  First cut.

*/

#include <udpinc.h>

/* RISC/OS is not returning the correct value when the network
   would block.  Force the "correct" value into EWOULDBLOCK so
   that the code works correctly.  */

#ifdef RISCOS
#undef EWOULDBLOCK
#define EWOULDBLOCK 11
#endif

int process_pipe(users, pipe_fd)
udp_data *users;
int pipe_fd;
{
    char inbuf[4096];
    int p_rval, rval, stat; 
   
    dbgout("Processing pipe data");
    
    /* Read the first chunk of data from the pipe here and then start 
       transferring data.  Note that the first read size could be zero
       bytes.  If this is the case, just read again.  */
    
    p_rval = read_data(pipe_fd, inbuf, sizeof(inbuf));
    dprintf2(dbgbuf, "Initial pipe read: %d, errno %d", p_rval, errno);
    dbgout(dbgbuf);

    /* Now form the connection to the server since we have data we want
       to print. */
    
    while(TRUE) { 
	users->flags &= ~(RETRY|CONNECTED);
	users->flags |= ALLOW_QUE;
	while(TRUE){
	    dprintf(dbgbuf, "Attempting to connect to service, flags field: %x", users->flags);
	    dbgout(dbgbuf);
	    rval = connect_service(users);
	    if(rval < OK){
		errs(users, RTEL_NOCONN, "process_pipe:badcon");
		return(rval);
	    }
	    if(rval == OK) break;
	    waiting(users);
	}

	/* We are now connected to the server.  Set the NDELAY option
	   on the network socket.  This will allow us to watch for the
	   case where forward progress has stopped and do something
	   to inform the user.  
	   
	   Note that IBM has to do things differently.  The fcntl call
	   does not return an invalid status, but it just doesn't work
	   on AIX machines.  */
	
#ifdef AIX
	rval = 1;
	stat = ioctl(users->tcp_fd, FIONBIO, &rval);
	if (stat < OK){
	    errs(users, errno, "process_file:delaynset");
	    return(PROC_ERR);
	}
#else
	stat = fcntl(users->tcp_fd, F_SETFL, O_NDELAY);
	if (stat < OK){
	    errs(users, errno, "process_file:delaynset");
	    return(PROC_ERR);
	}
#endif    
	
	/* Now start sending out the data */

	while(TRUE){
	    if(p_rval <= 0){
		if(p_rval == 0) break;
		errs(users, errno, "process_pipe:piperead");
		return(PROC_ERR);
	    }
	    
	    rval = write_data(users, inbuf, p_rval);
	    if(rval < 0) break;
	    
	    p_rval = read(pipe_fd, inbuf, sizeof(inbuf));
	    dprintf2(dbgbuf, "Pipe read2: %d, errno %d", p_rval, errno);
	    dbgout(dbgbuf);
    	}
    
	/* Close the network fd's to free up the service and spin on eof
	   while waiting for the next job */
	
	close(users->tcp_fd);
	close(users->udp_fd);
	dbgout("Closing network connection");
	
	p_rval = read_data(pipe_fd, inbuf, sizeof(inbuf));
	dprintf2(dbgbuf, "Pipe read3: %d, errno %d", p_rval, errno);
	dbgout(dbgbuf);
    }
}



write_data(users, buf, bufcnt)
udp_data *users;
char     *buf;
int      bufcnt; 
{
    char *ebp, expand_buf[8192];
    int bytes_togo, i, signaled, trycnt;
    
    /* Some people expect <lf> to be expanded to <lf><cr> even in pipes. */
    
    if(users->flags & LFCR_PIPE){
	ebp = expand_buf;
	for(i=0; i < bufcnt; i++){
	    switch ((unsigned char) *buf){
		case 0x0a: {
		    *ebp++ = 0x0d;
		    *ebp++ = *buf++;
		    break;
		}
		default: {
		    *ebp++ = *buf++;
		    break;
		}
	    }
	}
	bytes_togo = ebp - expand_buf;
	buf = expand_buf;
    }
    else {
	bytes_togo = bufcnt;
    }
    
    signaled = FALSE;
    trycnt = 0;

    while(bytes_togo != 0){
        bufcnt = write(users->tcp_fd, buf, bytes_togo); 
        dprintf2(dbgbuf, "Net write: %d, errno %d", bufcnt, errno);
        dbgout(dbgbuf);
	
        if(bufcnt < 0){
	    switch(errno){
		case EWOULDBLOCK:{
		    trycnt++;
		    if(trycnt > NO_PROGRESS){
			if(trycnt > ABORT_JOB){
			    dbgout("Aborting job due to no progress");
			}
			if(!signaled){
			    dbgout("Signaling no job progress");
			    signaled = TRUE;
			}
		    }
		    dbgout("Sleeping for one second");
		    sleep(1);
		    continue;
		}
		default:{
		    errs(users, errno, "process_pipe:wrtfailed");
		    return(PROC_ERR);
		}
	    }
	}
	else{
	    bytes_togo -= bufcnt;
	    buf += bufcnt;
	    if(bytes_togo > 0){
		trycnt++;
		sleep(1);
	    }
	}
    }
    return(OK);
}



read_data(fd, buffer, bufcnt) 
int fd;
char *buffer;
int bufcnt;
{
    int stat;

    while(TRUE){
	stat = read(fd, buffer, bufcnt);
	if(stat > 0){
	    return(stat);
	}
	sleep(3);
    }
}
