/***************************************************************\
*                                                              *
*                   Copyright (c) 1991 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: rteld                                          *
*       Created: Thurs 9 May 10 17:51 1991                     *
*                                                              *
*       Abstract:                                              *
*         This is the user interface to the wonderful world    *
*         of rtel's named pipe communications devices.         *
*                                                              *
*         There are five major command catagories this program *
*         handles:                                             *
*                                                              *
*           TALK --- This option moves data between a host     *
*                    pipe device and a port on a server.       *
*                                                              *
*           SETUP -- This option forks off a daemon process    *
*                    for each line in the server_hosts file    *
*                    that starts with a "/".  This would       *
*                    typically be used in a system startup     *
*                    file.                                     *
*                                                              *
*           KILL --- This option deletes currently running     *
*                    rtelpd processes.                         *
*                                                              *
*           REVIEW - This option shows what rtelpd process are *
*                    currently running and lists the named     *
*                    pipe device they are attached to.         *
*                                                              *
*           VERS --- This option will simply display the rtel  *
*                    version level.                            *
*                                                              *
*       Input:                                                 *
*         The user passes necessary parameters through the     *
*         startup command line.                                *
*                                                              *
*       Output:                                                *
*         If no errors occur, this routine will generate no    *
*         output.  If a processing error occurs, appropriate   *
*         error messages will be logged to the /usr/spool/rtel *
*         job_status file and also to the invoking terminal    *
*         if applicable.                                       *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  13-may-94  Completely removed do_cleanup, as it is no
                   longer required since kill files no longer
                   exist.

   RKL  19-jan-94  Create the job_status file if it doesn't
                   already exist.

   RKL  13-sep-93  Added -e option to expand <lf> into <cr><lf>
                   for pipe devices.

   RKL  31-jul-92  Replace bzero with rtel_zero.  Amazing how many
                   support calls such a small thing can generate.

   RKL  13-jul-92 Added doing automatic cleanup when doing a
                  system startup command.

   RKL  06-jun-92 Added debugging info.

   RKL  29-jun-92 Added -v option to show rtel version level.

   RKL  11-apr-92 Added -i (immediate) option.

   KCS  29-oct-91 added new option -t, which causes telnet
                  IAC padding and interpretation to work.

   KCS  10-sep-91 The parser complained because the users
                  structure was not initialized. 
			    
*/

#include <udpinc.h>

main(argc,argv)
int argc;
unsigned char* argv[];
{
    udp_data users;
    unsigned short euid;
    int process;

    /* Zero the users structure and parse the users input. */

    rtel_zero(&users, sizeof (udp_data));
    users.tcp_socket = SOCK_RTEL;

    /* Create the status file if it doesn't exist.  Note that if
       we can't create the file simply log a message to the
       console and continue processing.  */

    if (open_jobstat_fil()) {
	errs(&users, RTEL_CNTOPNJOB, "rtelpd:main");
	wrerr(&users,CONSOLE);
    }
    
    process = rteld_parse(argc, argv, &users);
    
    switch (process){
	case TALK:{
	    do_talk(&users);
	    break;
	}
	
	case SETUP:{
	    if (users.host[0] || users.svcnm[0] || users.pipe[0]){
		printf("Invalid option specified along with setup\r\n");
	    }
	    else{
		if ((euid = geteuid()) != 0) {
		    printf("You must have root privs for this option\r\n");
		}
		else{
		    do_setup(&users);
		}
	    }
	    break;
	}
	
	case KILL:{
	    if (users.host[0] || users.svcnm[0] || !users.pipe[0]){
		printf("Invalid option specified along with kill\r\n");
	    }
	    else{
		do_kill(&users);
	    }
	    break;
	}
	
	case REVIEW:{
	    if (users.host[0] || users.svcnm[0] || users.pipe[0]){
		printf("Invalid option specified along with kill\r\n");
	    }
	    else{
		do_review(&users);
	    }
	    break;
	}
	
	case VERS:{
	    printf("\r\n\nRTEL Version %s\r\n\n", VERSION);
	    break;
	}
	
	default:{
	    printf("Typical command line formats\r\n");
	    printf("----------------------------\r\n");
	    printf("rtelpd -cservice@host [options]\r\n");
	    printf("rtelpd -npipe_name [options]\r\n");
	    printf("rtelpd -c:pnum@host -npipe_name [options]\r\n");
	    printf("rtelpd -kall\r\n");
	    printf("rtelpd -kpipe_name\r\n");
	    printf("\r\n");
	    printf("Commands and Options\r\n");
	    printf("--------------------\r\n");
	    printf("-b      spawn rtelpd as a system daemon\r\n");
	    printf("-c      specify service/server or port number/server\r\n");
	    printf("-e      expand <lf> into <lf><cr>\r\n");
	    printf("-h      display this help message\r\n");
	    printf("-k      kill rtelpd for specified device (or all devices)\r\n");
            printf("-n      use the specified named pipe\r\n");
	    printf("-p      specify password to use with connection\r\n");
	    printf("-r      display currently running rtelpd daemons\r\n");
	    printf("-s      start RTEL daemons as specified in server_hosts\r\n");
	}
    }
    
    /* Check for errors. */
    if (users.next > 0){ 
	wrerr(&users, RTEL);
    }
    exit (0);
}
